@@ -624,8 +624,13 @@ gimp_iscissors_tool_button_release (GimpTool        *tool,
 		{
 		  curve = (ICurve *) iscissors->curves->data;
 
-		  if (abs (iscissors->x - curve->x1) < POINT_HALFWIDTH &&
-		      abs (iscissors->y - curve->y1) < POINT_HALFWIDTH)
+		  if (gimp_draw_tool_on_handle (GIMP_DRAW_TOOL (tool), gdisp,
+                                                iscissors->x, iscissors->y,
+                                                GIMP_HANDLE_CIRCLE,
+                                                curve->x1, curve->y1,
+                                                POINT_WIDTH, POINT_WIDTH,
+                                                GTK_ANCHOR_CENTER,
+                                                FALSE))
 		    {
 		      iscissors->x = curve->x1;
 		      iscissors->y = curve->y1;
@@ -1731,6 +1736,9 @@ gradmap_tile_validate (TileManager *tm,
   dw = tile_ewidth (tile);
   dh = tile_eheight (tile);
 
+  gimp_projection_finish_draw (gimage->projection);
+  gimp_projection_flush_now (gimage->projection);
+
   /* get corresponding tile in the gimage */
   srctile = tile_manager_get_tile (gimp_projection_get_tiles (gimage->projection),
 				   x, y, TRUE, FALSE);
