@@ -38,10 +38,10 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include <gtk/gtk.h>
 
@@ -87,42 +87,33 @@ static XBMSaveVals xsvals =
   "_mask"
 };
 
-typedef struct _XBMSaveInterface
-{
-  gint run;
-} XBMSaveInterface;
-
-static XBMSaveInterface xsint =
-{
-  FALSE			/* run */
-};
-
 
 /* Declare some local functions.
  */
 static void   query   (void);
-static void   run     (gchar      *name,
-		       gint        nparams,
-		       GimpParam  *param,
-		       gint       *nreturn_vals,
-		       GimpParam **return_vals);
+static void   run     (const gchar      *name,
+		       gint              nparams,
+		       const GimpParam  *param,
+		       gint             *nreturn_vals,
+		       GimpParam       **return_vals);
 
-static gint32 load_image              (gchar     *filename);
-static gint   save_image              (gchar     *filename,
-				       gchar     *prefix,
-				       gchar     *comment,
-				       gboolean   save_mask,
-				       gint32     image_ID, 
-				       gint32     drawable_ID);
-static gint   save_dialog             (gint32     drawable_ID);
-static void   save_ok_callback        (GtkWidget *widget,
-				       gpointer   data);
-static void   comment_entry_callback  (GtkWidget *widget,
-				       gpointer   data);
-static void   prefix_entry_callback   (GtkWidget *widget,
-				       gpointer   data);
-static void   mask_ext_entry_callback (GtkWidget *widget,
-				       gpointer   data);
+static gint32    load_image              (const gchar *filename);
+static gint      save_image              (const gchar *filename,
+                                          const gchar *prefix,
+                                          const gchar *comment,
+                                          gboolean     save_mask,
+                                          gint32       image_ID,
+                                          gint32       drawable_ID);
+static gboolean  save_dialog             (gint32       drawable_ID);
+#if 0
+/* DISABLED - see http://bugzilla.gnome.org/show_bug.cgi?id=82763 */
+static void      comment_entry_callback  (GtkWidget   *widget,
+                                          gpointer     data);
+#endif
+static void      prefix_entry_callback   (GtkWidget   *widget,
+                                          gpointer     data);
+static void      mask_ext_entry_callback (GtkWidget   *widget,
+                                          gpointer     data);
 
 GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -147,14 +138,11 @@ query (void)
     { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
     { GIMP_PDB_STRING, "raw_filename", "The name entered" }
   };
-  static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
 
   static GimpParamDef load_return_vals[] =
   {
     { GIMP_PDB_IMAGE,  "image",        "Output image" }
   };
-  static gint nload_return_vals = (sizeof (load_return_vals) /
-				   sizeof (load_return_vals[0]));
 
   static GimpParamDef save_args[] =
   {
@@ -171,7 +159,6 @@ query (void)
     { GIMP_PDB_INT32,    "write_mask",     "(0 = ignore, 1 = save as extra file)" },
     { GIMP_PDB_STRING,   "mask_extension", "Extension of the mask file" }
   } ;
-  static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
   gimp_install_procedure ("file_xbm_load",
                           "Load a file in X10 or X11 bitmap (XBM) file format",
@@ -179,76 +166,85 @@ query (void)
                           "Gordon Matzigkeit",
                           "Gordon Matzigkeit",
                           "1998",
-                          "<Load>/XBM",
+                          N_("X BitMap image"),
 			  NULL,
                           GIMP_PLUGIN,
-                          nload_args, nload_return_vals,
+                          G_N_ELEMENTS (load_args),
+                          G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
+  gimp_register_file_handler_mime ("file_xbm_load", "image/x-xbitmap");
+  gimp_register_load_handler ("file_xbm_load",
+			      "xbm,icon,bitmap",
+			      "");
+
   gimp_install_procedure ("file_xbm_save",
                           "Save a file in X10 or X11 bitmap (XBM) file format",
                           "Save a file in X10 or X11 bitmap (XBM) file format.  XBM is a lossless format for flat black-and-white (two color indexed) images.",
 			  "Gordon Matzigkeit",
                           "Gordon Matzigkeit",
                           "1998",
-                          "<Save>/XBM",
+                          N_("X BitMap image"),
 			  "INDEXED",
                           GIMP_PLUGIN,
-                          nsave_args, 0,
+                          G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
 
-  gimp_register_load_handler ("file_xbm_load",
-			      "xbm,icon,bitmap",
-			      "");
-  gimp_register_save_handler ("file_xbm_save",
-			      "xbm,icon,bitmap",
-			      "");
+  gimp_register_file_handler_mime ("file_xbm_save", "image/x-xbitmap");
+  gimp_register_save_handler ("file_xbm_save", "xbm,icon,bitmap", "");
 }
 
 static gchar *
-init_prefix (gchar *filename)
+init_prefix (const gchar *filename)
 {
   gchar *p, *prefix;
   gint len;
 
-  prefix = g_basename (filename);
-
-  /* Strip any extension. */
-  p = strrchr (prefix, '.');
-  if (p && p != prefix)
-    len = MIN (MAX_PREFIX, p - prefix);
-  else
-    len = MAX_PREFIX;
+  prefix = g_path_get_basename (filename);
 
   memset (xsvals.prefix, 0, sizeof (xsvals.prefix));
-  strncpy (xsvals.prefix, prefix, len);
+
+  /* Strip any extension. */
+  if (prefix)
+    {
+      p = strrchr (prefix, '.');
+      if (p && p != prefix)
+        len = MIN (MAX_PREFIX, p - prefix);
+      else
+        len = MAX_PREFIX;
+
+      strncpy (xsvals.prefix, prefix, len);
+      g_free (prefix);
+    }
 
   return xsvals.prefix;
 }
 
 static void
-run (gchar      *name,
-     gint        nparams,
-     GimpParam  *param,
-     gint       *nreturn_vals,
-     GimpParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
   static GimpParam   values[2];
-  GimpRunModeType    run_mode;
+  GimpRunMode        run_mode;
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
   gint32             image_ID;
   gint32             drawable_ID;
-  GimpParasite      *parasite = NULL; 
+  GimpParasite      *parasite = NULL;
   gchar             *mask_filename = NULL;
-  GimpExportReturnType export = GIMP_EXPORT_CANCEL;
+  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
+
+  INIT_I18N ();
 
-  INIT_I18N_UI();
   strncpy (xsvals.comment, _("Created with The GIMP"), MAX_COMMENT);
 
   run_mode = param[0].data.d_int32;
 
   *nreturn_vals = 1;
   *return_vals  = values;
+
   values[0].type          = GIMP_PDB_STATUS;
   values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
@@ -274,24 +270,25 @@ run (gchar      *name,
     }
   else if (strcmp (name, "file_xbm_save") == 0)
     {
-      image_ID = param[1].data.d_int32;
+      image_ID    = param[1].data.d_int32;
       drawable_ID = param[2].data.d_int32;
 
-      /*  eventually export the image */ 
       switch (run_mode)
 	{
 	case GIMP_RUN_INTERACTIVE:
 	case GIMP_RUN_WITH_LAST_VALS:
 	  gimp_ui_init ("xbm", FALSE);
 	  export = gimp_export_image (&image_ID, &drawable_ID, "XBM",
-				      GIMP_EXPORT_CAN_HANDLE_INDEXED |
-				      GIMP_EXPORT_CAN_HANDLE_ALPHA );
+				      GIMP_EXPORT_CAN_HANDLE_BITMAP |
+				      GIMP_EXPORT_CAN_HANDLE_ALPHA);
+
 	  if (export == GIMP_EXPORT_CANCEL)
 	    {
 	      values[0].data.d_status = GIMP_PDB_CANCEL;
 	      return;
-	  }
+            }
 	  break;
+
 	default:
 	  break;
 	}
@@ -374,13 +371,10 @@ run (gchar      *name,
 
           if (parasite)
     	    {
-	      gpointer data;
-	      gint     size;
+	      gint size = gimp_parasite_data_size (parasite);
 
-	      data = gimp_parasite_data (parasite);
-	      size = gimp_parasite_data_size (parasite);
-
-	      strncpy (xsvals.comment, data, MIN (size, MAX_COMMENT));
+	      strncpy (xsvals.comment,
+		       gimp_parasite_data (parasite), MIN (size, MAX_COMMENT));
 	      xsvals.comment[MIN (size, MAX_COMMENT) + 1] = 0;
 
 	      gimp_parasite_free (parasite);
@@ -390,12 +384,9 @@ run (gchar      *name,
 
           if (parasite)
 	    {
-	      gpointer data;
-	      gint     x, y;
+	      gint x, y;
 
-	      data = gimp_parasite_data (parasite);
-
-	      if (sscanf (data, "%i %i", &x, &y) == 2)
+	      if (sscanf (gimp_parasite_data (parasite), "%i %i", &x, &y) == 2)
 	       {
 	         xsvals.use_hot = TRUE;
 	         xsvals.x_hot = x;
@@ -417,7 +408,7 @@ run (gchar      *name,
 
 	  temp = mask_filename;
 
-	  if ((dirname = g_dirname (param[3].data.d_string)) != NULL)
+	  if ((dirname = g_path_get_dirname (param[3].data.d_string)) != NULL)
 	    {
 	      mask_filename = g_strdup_printf ("%s/%s%s.xbm",
 					       dirname, temp, xsvals.mask_ext);
@@ -435,18 +426,19 @@ run (gchar      *name,
 	  temp = xsvals.prefix;
 	  while (*temp)
 	    {
-	      if (!isalnum (*temp))
+	      if (!g_ascii_isalnum (*temp))
 		*temp = '_';
 	      temp ++;
 	    }
 
-	  mask_prefix = g_strdup_printf ("%s%s", xsvals.prefix, xsvals.mask_ext);
+	  mask_prefix = g_strdup_printf ("%s%s",
+                                         xsvals.prefix, xsvals.mask_ext);
 
 	  if (save_image (param[3].data.d_string,
 			  xsvals.prefix,
 			  xsvals.comment,
 			  FALSE,
-			  image_ID, drawable_ID) 
+			  image_ID, drawable_ID)
               && (!xsvals.write_mask || save_image (mask_filename,
                                                     mask_prefix,
                                                     xsvals.comment,
@@ -479,11 +471,11 @@ run (gchar      *name,
 
 /* Return the value of a digit. */
 static gint
-getval (gint c, 
+getval (gint c,
 	gint base)
 {
-  static guchar *digits = "0123456789abcdefABCDEF";
-  gint val;
+  const gchar *digits = "0123456789abcdefABCDEF";
+  gint         val;
 
   /* Include uppercase hex digits. */
   if (base == 16)
@@ -555,7 +547,7 @@ fgetcomment (FILE *fp)
 		  return NULL;
 		}
 	    }
-	  else if (isspace (c))
+	  else if (c != EOF && g_ascii_isspace (c))
 	    {
 	      /* Skip leading whitespace */
 	      continue;
@@ -620,7 +612,7 @@ cpp_fgetc (FILE *fp)
 
 /* Match a string with a file. */
 static gint
-match (FILE  *fp, 
+match (FILE  *fp,
        gchar *s)
 {
   gint c;
@@ -652,7 +644,7 @@ get_int (FILE *fp)
 
   do
     c = cpp_fgetc (fp);
-  while (c != EOF && !isdigit (c));
+  while (c != EOF && ! g_ascii_isdigit (c));
 
   if (c == EOF)
     return 0;
@@ -666,7 +658,7 @@ get_int (FILE *fp)
 	  c = fgetc (fp);
 	  base = 16;
 	}
-      else if (isdigit (c))
+      else if (g_ascii_isdigit (c))
 	base = 8;
       else
 	{
@@ -696,7 +688,7 @@ get_int (FILE *fp)
 
 
 static gint
-load_image (gchar *filename)
+load_image (const gchar *filename)
 {
   FILE *fp;
   gint32 image_ID, layer_ID;
@@ -724,11 +716,13 @@ load_image (gchar *filename)
   fp = fopen (filename, "rb");
   if (!fp)
     {
-      g_message (_("XBM: cannot open \"%s\"\n"), filename);
+      g_message (_("Could not open '%s' for reading: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
       return -1;
     }
 
-  name_buf = g_strdup_printf (_("Loading %s:"), filename);
+  name_buf = g_strdup_printf (_("Opening '%s'..."),
+                              gimp_filename_to_utf8 (filename));
   gimp_progress_init (name_buf);
   g_free (name_buf);
 
@@ -739,12 +733,12 @@ load_image (gchar *filename)
   c = ' ';
   do
     {
-      if (isspace (c))
+      if (g_ascii_isspace (c))
 	{
 	  if (match (fp, "char"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  intbits = 8;
 		  continue;
@@ -753,7 +747,7 @@ load_image (gchar *filename)
 	  else if (match (fp, "short"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  intbits = 16;
 		  continue;
@@ -766,7 +760,7 @@ load_image (gchar *filename)
 	  if (match (fp, "width"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  width = get_int (fp);
 		  continue;
@@ -775,7 +769,7 @@ load_image (gchar *filename)
 	  else if (match (fp, "height"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  height = get_int (fp);
 		  continue;
@@ -784,7 +778,7 @@ load_image (gchar *filename)
 	  else if (match (fp, "x_hot"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  x_hot = get_int (fp);
 		  continue;
@@ -793,7 +787,7 @@ load_image (gchar *filename)
 	  else if (match (fp, "y_hot"))
 	    {
 	      c = fgetc (fp);
-	      if (isspace (c))
+	      if (g_ascii_isspace (c))
 		{
 		  y_hot = get_int (fp);
 		  continue;
@@ -807,25 +801,29 @@ load_image (gchar *filename)
 
   if (c == EOF)
     {
-      g_message (_("XBM: cannot read header (ftell == %ld)\n"), ftell (fp));
+      g_message (_("'%s':\nCould not read header (ftell == %ld)"),
+                 gimp_filename_to_utf8 (filename), ftell (fp));
       return -1;
     }
 
   if (width == 0)
     {
-      g_message (_("XBM: no image width specified\n"));
+      g_message (_("'%s':\nNo image width specified"),
+                 gimp_filename_to_utf8 (filename));
       return -1;
     }
 
   if (height == 0)
     {
-      g_message (_("XBM: no image height specified\n"));
+      g_message (_("'%s':\nNo image height specified"),
+                 gimp_filename_to_utf8 (filename));
       return -1;
     }
 
   if (intbits == 0)
     {
-      g_message (_("XBM: no image data type specified\n"));
+      g_message (_("'%s':\nNo image data type specified"),
+                 gimp_filename_to_utf8 (filename));
       return -1;
     }
 
@@ -863,7 +861,7 @@ load_image (gchar *filename)
     }
 
   /* Set a black-and-white colormap. */
-  gimp_image_set_cmap (image_ID, cmap, 2);
+  gimp_image_set_colormap (image_ID, cmap, 2);
 
   layer_ID = gimp_layer_new (image_ID,
 			     _("Background"),
@@ -930,16 +928,16 @@ load_image (gchar *filename)
 }
 
 static gboolean
-save_image (gchar    *filename,
-	    gchar    *prefix,
-	    gchar    *comment,
-	    gboolean  save_mask,
-	    gint32    image_ID,
-	    gint32    drawable_ID)
+save_image (const gchar *filename,
+	    const gchar *prefix,
+	    const gchar *comment,
+	    gboolean     save_mask,
+	    gint32       image_ID,
+	    gint32       drawable_ID)
 {
   GimpDrawable *drawable;
   GimpPixelRgn  pixel_rgn;
-  FILE *fp;
+  FILE         *fp;
 
   gint width, height, colors, dark;
   gint intbits, lineints, need_comma, nints, rowoffset, tileheight;
@@ -948,19 +946,20 @@ save_image (gchar    *filename,
   gboolean has_alpha;
   gint     bpp;
 
-  guchar *data, *cmap, *name_buf, *intfmt;
+  guchar *data, *cmap;
+  gchar  *name_buf, *intfmt;
 
   drawable = gimp_drawable_get (drawable_ID);
-  width  = drawable->width;
-  height = drawable->height;
-  cmap = gimp_image_get_cmap (image_ID, &colors);
+  width    = drawable->width;
+  height   = drawable->height;
+  cmap     = gimp_image_get_colormap (image_ID, &colors);
 
-  if (!gimp_drawable_is_indexed (drawable_ID) || colors > 2)
+  if (! gimp_drawable_is_indexed (drawable_ID) || colors > 2)
     {
       /* The image is not black-and-white. */
-      g_message (_("The image which you are trying to save as\n"
+      g_message (_("The image which you are trying to save as "
 		   "an XBM contains more than two colors.\n\n"
-		   "Please convert it to a black and white\n"
+		   "Please convert it to a black and white "
 		   "(1-bit) indexed image and try again."));
       return FALSE;
     }
@@ -976,10 +975,6 @@ save_image (gchar    *filename,
 
   bpp = gimp_drawable_bpp (drawable_ID);
 
-  name_buf = g_strdup_printf (_("Saving %s:"), filename);
-  gimp_progress_init (name_buf);
-  g_free (name_buf);
-
   /* Figure out which color is black, and which is white. */
   dark = 0;
   if (colors > 1)
@@ -998,13 +993,23 @@ save_image (gchar    *filename,
   fp = fopen (filename, "w");
   if (!fp)
     {
-      g_message (_("XBM: cannot create \"%s\"\n"), filename);
+      g_message (_("Could not open '%s' for writing: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
       return FALSE;
     }
 
+  name_buf = g_strdup_printf (_("Saving '%s'..."),
+                              gimp_filename_to_utf8 (filename));
+  gimp_progress_init (name_buf);
+  g_free (name_buf);
+
   /* Maybe write the image comment. */
+#if 0
+  /* DISABLED - see http://bugzilla.gnome.org/show_bug.cgi?id=82763 */
+  /* a future version should write the comment at the end of the file */
   if (*comment)
     fprintf (fp, "/* %s */\n", comment);
+#endif
 
   /* Write out the image height and width. */
   fprintf (fp, "#define %s_width %d\n",  prefix, width);
@@ -1127,7 +1132,7 @@ save_image (gchar    *filename,
   return TRUE;
 }
 
-static gint
+static gboolean
 save_dialog (gint32 drawable_ID)
 {
   GtkWidget *dlg;
@@ -1138,138 +1143,138 @@ save_dialog (gint32 drawable_ID)
   GtkWidget *entry;
   GtkWidget *spinbutton;
   GtkObject *adj;
+  gboolean   run;
 
   dlg = gimp_dialog_new (_("Save as XBM"), "xbm",
-			 gimp_standard_help_func, "filters/xbm.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+                         NULL, 0,
+			 gimp_standard_help_func, "file-xbm-save",
 
-			 _("OK"), save_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                         GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
-			 NULL);
-
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
+                         NULL);
 
   /* parameter settings */
-  frame = gtk_frame_new (_("XBM Options"));
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
+  frame = gimp_frame_new (_("XBM Options"));
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  vbox = gtk_vbox_new (FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
+  vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_add (GTK_CONTAINER (frame), vbox);
 
   /*  X10 format  */
-  toggle = gtk_check_button_new_with_label (_("X10 Format Bitmap"));
+  toggle = gtk_check_button_new_with_mnemonic (_("_X10 format bitmap"));
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &xsvals.x10_format);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), xsvals.x10_format);
   gtk_widget_show (toggle);
 
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &xsvals.x10_format);
+
   table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
   gtk_widget_show (table);
 
   /* prefix */
-  entry = gtk_entry_new_with_max_length (MAX_PREFIX);
+  entry = gtk_entry_new ();
+  gtk_entry_set_max_length (GTK_ENTRY (entry), MAX_PREFIX);
   gtk_entry_set_text (GTK_ENTRY (entry), xsvals.prefix);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("Identifier Prefix:"), 1.0, 0.5,
+			     _("_Identifier prefix:"), 0.0, 0.5,
 			     entry, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-                      GTK_SIGNAL_FUNC (prefix_entry_callback),
-                      NULL);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (prefix_entry_callback),
+                    NULL);
 
   /* comment string. */
-  entry = gtk_entry_new_with_max_length (MAX_COMMENT);
-  gtk_widget_set_usize (entry, 240, 0);
+#if 0
+  /* DISABLED - see http://bugzilla.gnome.org/show_bug.cgi?id=82763 */
+  entry = gtk_entry_new ();
+  gtk_entry_set_max_length (GTK_ENTRY (entry), MAX_COMMENT);
+  gtk_widget_set_size_request (entry, 240, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), xsvals.comment);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Comment:"), 1.0, 0.5,
+			     _("Comment:"), 0.0, 0.5,
 			     entry, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-                      GTK_SIGNAL_FUNC (comment_entry_callback),
-                      NULL);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (comment_entry_callback),
+                    NULL);
+#endif
 
   /* hotspot toggle */
-  toggle = gtk_check_button_new_with_label (_("Write Hot Spot Values"));
+  toggle = gtk_check_button_new_with_mnemonic (_("_Write hot spot values"));
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &xsvals.use_hot);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), xsvals.use_hot);
   gtk_widget_show (toggle);
 
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &xsvals.use_hot);
+
   table = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
   gtk_widget_show (table);
 
-  gtk_object_set_data (GTK_OBJECT (toggle), "set_sensitive", table);
+  g_object_set_data (G_OBJECT (toggle), "set_sensitive", table);
   gtk_widget_set_sensitive (table, xsvals.use_hot);
 
   spinbutton = gimp_spin_button_new (&adj, xsvals.x_hot, 0,
 				     gimp_drawable_width (drawable_ID) - 1,
 				     1, 1, 1, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
-		      &xsvals.x_hot);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("Hot Spot X:"), 1.0, 0.5,
+			     _("Hot spot _X:"), 0.0, 0.5,
 			     spinbutton, 1, TRUE);
+  g_signal_connect (adj, "value_changed",
+                    G_CALLBACK (gimp_int_adjustment_update),
+                    &xsvals.x_hot);
 
   spinbutton = gimp_spin_button_new (&adj, xsvals.y_hot, 0,
 				     gimp_drawable_height (drawable_ID) - 1,
 				     1, 1, 1, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
-		      &xsvals.y_hot);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Y:"), 1.0, 0.5,
+			     _("Hot spot _Y:"), 0.0, 0.5,
 			     spinbutton, 1, TRUE);
+  g_signal_connect (adj, "value_changed",
+                    G_CALLBACK (gimp_int_adjustment_update),
+                    &xsvals.y_hot);
 
   /* mask file */
-  frame = gtk_frame_new (_("Mask File"));
+  frame = gimp_frame_new (_("Mask File"));
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
   table = gtk_table_new (2, 2, FALSE);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 2);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_widget_show (table);
 
-  toggle = gtk_check_button_new_with_label (_("Write Extra Mask File"));
+  toggle = gtk_check_button_new_with_mnemonic (_("W_rite extra mask file"));
   gtk_table_attach_defaults (GTK_TABLE (table), toggle, 0, 2, 0, 1);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &xsvals.write_mask);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), xsvals.write_mask);
   gtk_widget_show (toggle);
 
-  entry = gtk_entry_new_with_max_length (MAX_MASK_EXT);
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &xsvals.write_mask);
+
+  entry = gtk_entry_new ();
+  gtk_entry_set_max_length (GTK_ENTRY (entry), MAX_MASK_EXT);
   gtk_entry_set_text (GTK_ENTRY (entry), xsvals.mask_ext);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Mask File Extension:"), 1.0, 0.5,
+			     _("_Mask file extension:"), 0.0, 0.5,
 			     entry, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-                      GTK_SIGNAL_FUNC (mask_ext_entry_callback),
-                      NULL);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mask_ext_entry_callback),
+                    NULL);
 
-  gtk_object_set_data (GTK_OBJECT (toggle), "set_sensitive", entry);
+  g_object_set_data (G_OBJECT (toggle), "set_sensitive", entry);
   gtk_widget_set_sensitive (entry, xsvals.write_mask);
 
   gtk_widget_set_sensitive (frame, gimp_drawable_has_alpha (drawable_ID));
@@ -1278,14 +1283,16 @@ save_dialog (gint32 drawable_ID)
   gtk_widget_show (vbox);
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  return xsint.run;
+  gtk_widget_destroy (dlg);
+
+  return run;
 }
 
-
 /* Update the comment string. */
+#if 0
+/* DISABLED - see http://bugzilla.gnome.org/show_bug.cgi?id=82763 */
 static void
 comment_entry_callback (GtkWidget *widget,
 			gpointer   data)
@@ -1294,6 +1301,7 @@ comment_entry_callback (GtkWidget *widget,
   strncpy (xsvals.comment,
 	   gtk_entry_get_text (GTK_ENTRY (widget)), MAX_COMMENT);
 }
+#endif
 
 static void
 prefix_entry_callback (GtkWidget *widget,
@@ -1312,12 +1320,3 @@ mask_ext_entry_callback (GtkWidget *widget,
   strncpy (xsvals.mask_ext,
 	   gtk_entry_get_text (GTK_ENTRY (widget)), MAX_MASK_EXT);
 }
-
-static void
-save_ok_callback (GtkWidget *widget,
-		  gpointer   data)
-{
-  xsint.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
